#ifndef OM_COMMON_H
#define OM_COMMON_H

/**
 * Designation if this library.
 */
extern const char * OmLibraryDesignation;

/**
 * Version of this library.
 * {major}{minor}{stable=0/unstable=1}
 */
extern const char * OmLibraryVersion;

/**
 * Date of this library.
 * {year}{month}{day}
 */
extern const char * OmLibraryDating;

/**
 * Enum values describing the status of the inner OpenMath device.
 * They MUST be a subset from the C library to ensure full compatibility with casts.
 */
enum OmStatus
{
  OmSuccessStatus           = OMsuccess,
  OmFailedStatus            = OMfailed,
  OmNoMemoryStatus          = OMnoMem,
  OmSystemErrorStatus       = OMerrorSys,
  OmIOErrorStatus           = OMemptyIO,
  OmNoMoreTokenStatus       = OMnoMoreToken,
  OmTimeoutedReadStatus     = OMtimeoutedRead,
  OmMalformedInputStatus    = OMmalformedInput,
  OmAddressInUseStatus      = OMaddrInUse,
  OmConnectionFailedStatus  = OMconnectFailed,
  OmNotYetImplementedStatus = OMnotImplemented,
  OmInternalErrorStatus     = OMinternalError
};

/**
 * Enum values describing the types of the OpenMath objects.
 * They MUST be a subset from the C library to ensure full compatibility with casts.
 */
enum OmType
{
  OmIntegerType               = OMtokenInt32,
  OmBigIntegerType            = OMtokenBigInt,
  OmFloatType                 = OMtokenFloat64,
  OmByteArrayType             = OMtokenByteArray,
  OmVariableType              = OMtokenVar,
  OmStringType                = OMtokenString,
  OmWStringType               = OMtokenWCString,
  OmSymbolType                = OMtokenSymbol,
  OmPInstructionType          = OMtokenPInstruction,
  OmCommentType               = OMtokenComment,
  OmApplicationType           = OMtokenApp,
  OmEndApplicationType        = OMtokenEndApp,
  OmAttributeType             = OMtokenAttr,
  OmEndAttributeType          = OMtokenEndAttr,
  OmAttributeParameterType    = OMtokenAtp,
  OmEndAttributeParameterType = OMtokenEndAtp,
  OmErrorType                 = OMtokenError,
  OmEndErrorType              = OMtokenEndError,
  OmObjectType                = OMtokenObject,
  OmEndObjectType             = OMtokenEndObject,
  OmBindingType               = OMtokenBind,
  OmEndBindingType            = OMtokenEndBind,
  OmBindingVariableType       = OMtokenBVar,
  OmEndBindingVariableType    = OMtokenEndBVar,
  OmUnknownType               = OMtokenUnknown
};

/**
 * Enum values describing which encoding can be used during I/O operations.
 * They MUST be a subset from the C library to ensure full compatibility with casts.
 */
enum OmEncoding
{
  OmXmlEncoding    = OMencodingXML,
  OmBinaryEncoding = OMencodingBinary
};

#endif // OM_COMMON_H
