/* OpenMath C++ Library
 *
 * Copyright (C) 1999 - Christophe Magnani
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <string>
#include "OmHeaders.h"

using namespace std;

static void Help()
{
  cout << OmLibraryDesignation << " - Version " << OmLibraryVersion << " (" << OmLibraryDating << ")\n"
       << "\tBased on the INRIA C library\n"
       << "\tCopyright (C) 1999 - Christophe Magnani\n\n";

  cout << "This piping program is able to reformat standard input to standard output\n"
       << "Usage: pipe command < in_file > out_file\n";

  cout << "Description of command: \n"
       << "\thelp -- Get help\n"
       << "\txx   -- Translate from Xml to Xml\n"
       << "\txb   -- Translate from Xml to Binary\n"
       << "\tbx   -- Translate from Binary to Xml\n"
       << "\tbb   -- Translate from Binary to Binary\n\n";
}

static void Translate(OmEncoding inputEncoding_in, OmEncoding outputEncoding_in)
{
  try
    {
      OmInputFileStream inputStream;
      OmOutputFileStream outputStream;
      OmInputDevice inputDevice(inputStream, inputEncoding_in, false);
      OmOutputDevice outputDevice(outputStream, outputEncoding_in, false);
      OmDocumentNode document;
      (&document)
	->load(inputDevice)
	->save(outputDevice);
    }
  catch (OmException e)
    {
      cerr << "OpenMath exception caught: " << e.translate() << "\n";
    }
  catch (...)
    {
      cerr << "Serious error but unknown\n";
    }
}

static void Execute(string command_in)
{
  if (command_in=="help")
    { Help(); }
  else if (command_in=="xx")
    { Translate(OmXmlEncoding, OmXmlEncoding); }
  else if (command_in=="xb")
    { Translate(OmXmlEncoding, OmBinaryEncoding); }
  else if (command_in=="bx")
    { Translate(OmBinaryEncoding, OmXmlEncoding); }
  else if (command_in=="bb")
    { Translate(OmBinaryEncoding, OmBinaryEncoding); }
  else
    { Help(); }
}

int main(int argc, char * argv[])
{
  // Execute
  if (argc!=2)
    { Help(); }
  else if (argc==2)
    { Execute(argv[1]); }

  return 0;
}
